﻿#include "StdAfx.h"
#include "rhinoSdkPlugInDeclare.h"
#include "TestVakerPlugIn.h"
#include "Resource.h"

#pragma warning(push)
#pragma warning(disable : 4073)
#pragma init_seg(lib)
#pragma warning(pop)

// Rhino plug-in declarations
RHINO_PLUG_IN_DECLARE
RHINO_PLUG_IN_NAME(L"TestVaker");
RHINO_PLUG_IN_ID(L"094537D9-8380-4AD5-8D09-FC4BFE92B683");
RHINO_PLUG_IN_VERSION(__DATE__ "  " __TIME__)
RHINO_PLUG_IN_DESCRIPTION(L"TestVaker plug-in for Rhinoceros®");
RHINO_PLUG_IN_ICON_RESOURCE_ID(IDI_ICON);

// Rhino plug-in developer declarations
RHINO_PLUG_IN_DEVELOPER_ORGANIZATION(L"My Company Name");
RHINO_PLUG_IN_DEVELOPER_ADDRESS(L"123 Developer Street\r\nCity State 12345-6789");
RHINO_PLUG_IN_DEVELOPER_COUNTRY(L"My Country");
RHINO_PLUG_IN_DEVELOPER_PHONE(L"123.456.7890");
RHINO_PLUG_IN_DEVELOPER_FAX(L"123.456.7891");
RHINO_PLUG_IN_DEVELOPER_EMAIL(L"support@mycompany.com");
RHINO_PLUG_IN_DEVELOPER_WEBSITE(L"http://www.mycompany.com");
RHINO_PLUG_IN_UPDATE_URL(L"http://www.mycompany.com/support");

// The one and only CTestVakerPlugIn object
static class CTestVakerPlugIn thePlugIn;

/////////////////////////////////////////////////////////////////////////////
// CTestVakerPlugIn definition

CTestVakerPlugIn& TestVakerPlugIn()
{
	return thePlugIn;
}

CTestVakerPlugIn::CTestVakerPlugIn()
{
	m_plugin_version = RhinoPlugInVersion();
}

/////////////////////////////////////////////////////////////////////////////
// Required overrides

const wchar_t* CTestVakerPlugIn::PlugInName() const
{
	return RhinoPlugInName();
}

const wchar_t* CTestVakerPlugIn::PlugInVersion() const
{
	return m_plugin_version;
}

GUID CTestVakerPlugIn::PlugInID() const
{
	// {094537D9-8380-4AD5-8D09-FC4BFE92B683}
	return ON_UuidFromString(RhinoPlugInId());
}

/////////////////////////////////////////////////////////////////////////////
// Additional overrides

BOOL CTestVakerPlugIn::OnLoadPlugIn()
{
	return TRUE;
}

void CTestVakerPlugIn::OnUnloadPlugIn()
{
}

/////////////////////////////////////////////////////////////////////////////
// Document user data overrides

BOOL CTestVakerPlugIn::CallWriteDocument(const CRhinoFileWriteOptions& options)
{
  UNREFERENCED_PARAMETER(options);

  wchar_t computer_name[MAX_COMPUTERNAME_LENGTH + 1];
  memset(computer_name, 0, sizeof(computer_name));
  unsigned long computer_name_length = MAX_COMPUTERNAME_LENGTH + 1;
  if (!GetComputerName(computer_name, &computer_name_length))
    computer_name[0] = 0;
  m_computer_name = computer_name;

  wchar_t user_name[UNLEN + 1];
  memset(user_name, 0, sizeof(user_name));
  unsigned long user_name_length = UNLEN + 1;
  if (!GetUserNameW(user_name, &user_name_length))
    m_user_name[0] = 0;
  m_user_name = user_name;

  return TRUE;
}

BOOL CTestVakerPlugIn::WriteDocument(CRhinoDoc& doc, ON_BinaryArchive& archive, const CRhinoFileWriteOptions& options)
{
  UNREFERENCED_PARAMETER(doc);
  UNREFERENCED_PARAMETER(options);

  bool rc = archive.BeginWrite3dmChunk(TCODE_ANONYMOUS_CHUNK, 1, 0);
  if (!rc)
    return FALSE;

  for (;;)
  {
    // write version 1.0 fields

    rc = archive.WriteString(m_computer_name);
    if (!rc) break;

    rc = archive.WriteString(m_user_name);
    if (!rc) break;

    break;
  }

  // If ON_BinaryArchive::BeginWrite3dmChunk() returns true, then ON_BinaryArchive::EndWrite3dmChunk() 
  // must be called even if a read operation failed.
  if (!archive.EndWrite3dmChunk())
    rc = false;

  return rc ? TRUE : FALSE;
}

BOOL CTestVakerPlugIn::ReadDocument(CRhinoDoc& doc, ON_BinaryArchive& archive, const CRhinoFileReadOptions& options)
{
  UNREFERENCED_PARAMETER(doc);
  UNREFERENCED_PARAMETER(options);

  ResetData();

  int major_version = 0;
  int minor_version = 0;
  bool rc = archive.BeginRead3dmChunk(TCODE_ANONYMOUS_CHUNK, &major_version, &minor_version);
  if (!rc)
    return FALSE;

  for (;;)
  {
    rc = (major_version == 1 && minor_version == 0);
    if (!rc) break;

    // read version 1.0 fields

    rc = archive.ReadString(m_computer_name);
    if (!rc) break;

    rc = archive.ReadString(m_user_name);
    if (!rc) break;

    break;
  }

  // If ON_BinaryArchive::BeginRead3dmChunk() returns true, then ON_BinaryArchive::EndRead3dmChunk() 
  // must be called even if a read operation failed.
  if (!archive.EndRead3dmChunk())
    rc = false;

  return rc ? TRUE : FALSE;
}

void CTestVakerPlugIn::GetData(ON_wString& data)
{
  if (m_computer_name.IsNotEmpty() && m_user_name.IsNotEmpty())
  {
    data.Format(L"Document last saved by %s\\%s", 
      static_cast<const wchar_t*>(m_computer_name), 
      static_cast<const wchar_t*>(m_user_name)
      );
  }
}

void CTestVakerPlugIn::ResetData()
{
  m_computer_name.Destroy();
  m_user_name.Destroy();
}

void CTestVakerEventWatcher::OnCloseDocument(CRhinoDoc& doc)
{
  UNREFERENCED_PARAMETER(doc);
  TestVakerPlugIn().ResetData();
}
